<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

interface HasViewRouteInterface
{
    public function viewRoute(array $replacements = []): ?string;

    public function viewRouteTemplate(array $replacements = []): string;
}
